
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import java.util.List;
import java.util.Vector;

import org.aspectj.weaver.tools.GeneratedClassHandler;
import org.aspectj.weaver.tools.WeavingAdaptor;

public class TestStatic {

	/**
	 * @param args
	 */
	public static final String classPathProperty = "java.class.path";
	public static final String bootPath = "sun.boot.class.path";

	private URL[] getWeaversURL() {
		try {
			URL url = new File("aspects.jar").toURI().toURL();
			return new URL[] { url };
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return new URL[0];

	}

	private URL[] getClasspathURL() {
		List<URL> result = new Vector<URL>();
		for (String line : System.getProperty(classPathProperty).split(
				File.pathSeparator)) {
			try {
				System.out.println("Path element: " + line);
				result.add(new File(line).toURI().toURL());
			} catch (MalformedURLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		URL[] res = new URL[0];
		return result.toArray(res);

	}

	public static void main(String[] args) {
		System.setProperty(classPathProperty, System
				.getProperty(classPathProperty)
				+ File.pathSeparator
				+ "."
				+ File.pathSeparator
				+ System.getProperty(bootPath));
		TestStatic ts = new TestStatic();
		WeavingAdaptor weavingAdaptor = new WeavingAdaptor(
				new GeneratedClassHandler() {

					@Override
					public void acceptClass(final String arg0, final byte[] arg1) {
						System.out.println("Accepting for: " + arg0);
						new ClassLoader(ClassLoader.getSystemClassLoader()) {
							private void define() {
								super.defineClass(arg0, arg1, 0, arg1.length);
							}
						}.define();
					}
				}, ts.getClasspathURL(), ts.getWeaversURL());

		try {

			byte[] bytes = readEntireInputStream(new FileInputStream(args[0]));
			byte[] newBytes = weavingAdaptor.weaveClass("test.Test", bytes,
					true);
			ts.dumpWovenClass("test.Test", newBytes);

		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void dumpWovenClass(String classname, byte[] wovenclass) {
		try {
			FileOutputStream fos = new FileOutputStream("INSTR_" + classname
					+ ".class");
			ByteArrayInputStream bin = new ByteArrayInputStream(wovenclass);
			int bytesRead = 0;
			byte[] b = new byte[8192];
			while ((bytesRead = bin.read(b)) != -1)
				fos.write(b, 0, bytesRead);
			fos.close();
		} catch (Exception e) {
			System.err.println("e");
		}
	}

	public static byte[] readEntireInputStream(InputStream is) {
		try {
			return readToEnd(is);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	private static byte[] readToEnd(InputStream is) throws IOException {
		// create channel for input stream
		ReadableByteChannel bc = Channels.newChannel(is);
		ByteBuffer bb = ByteBuffer.allocate(READ_BLOCK);

		while (bc.read(bb) != -1) {
			bb = resizeBuffer(bb); // get new buffer for read
		}
		byte[] result = new byte[bb.position()];
		bb.position(0);
		bb.get(result);

		return result;
	}

	private static ByteBuffer resizeBuffer(ByteBuffer in) {
		ByteBuffer result = in;
		if (in.remaining() < READ_BLOCK) {
			// create new buffer
			result = ByteBuffer.allocate(in.capacity() * 2);
			// set limit to current position in buffer and set position to zero.
			in.flip();
			// put original buffer to new buffer
			result.put(in);
		}
		return result;
	}

	private static final int READ_BLOCK = 8192;

}
